/*global window, RSVP */
/*jslint nomen: true, indent: 2, maxerr: 3*/
(function (window, rJS, loopEventListener, jIO, mimeType) {
  "use strict";

  var CODE_CONTENT_KEY = "text_content";

  function getEditorContent(gadget) {
    return new RSVP.Queue()
    .push(function () {
      return gadget.getDeclaredGadget('codeeditor');
    })
    .push(function (code_editor_gadget) {
      return code_editor_gadget.getContent();
    });
  }

  function getFileContent(gadget) {
    var file_list = gadget.props.element
                          .querySelector("form.crib-editor-file-upload .file")
                          .files;
    if (file_list.length === 0) {
      throw "No file Provided, not saving";
    } else {
      return file_list[0];
    }
  }

  function saveTextContent(gadget, event) {
    var content,
      url = gadget.props.element.querySelector("form.crib-editor-get .url").value,
      mimetype = gadget.props.element.querySelector("form.crib-editor-save .mimetype").value;
    if (!mimetype) {
      mimetype = mimeType.lookup(url, false, "application/octet-stream");
    }
    return new RSVP.Queue()
    .push(function () {
      if (gadget.props.use_editor) {
        return getEditorContent(gadget);
      } else {
        return getFileContent(gadget);
      }
    })
    .push(function (data) {
      if (gadget.props.use_editor) {
        content = data[CODE_CONTENT_KEY] || "";
      } else {
        content = data;
      }
      return gadget.crib_sw_put(url, {content: content, type: mimetype});
    })
    .push(function () {
      if (url !== gadget.props.options.url) {
        return gadget.redirect({
          page: "editor",
          url: url,
          message: "Saved " + url + " files at " + Date()
        });
      }
      gadget.props.element
                  .querySelector("form.crib-editor-file-upload").reset();
      gadget.props.element.querySelector(".crib-editor-save-status")
        .textContent = "Saved " + url + " files at " + Date();
    })
    .push(undefined, function (error) {
      gadget.props.element.querySelector(".crib-editor-save-status")
        .textContent = error;
    });
  }

  function displayProperEditor(gadget) {
    if (gadget.props.use_editor) {
      gadget.props.element.querySelector("div.file-upload")
        .style = "display:none;";
      gadget.props.element.querySelector("div.editor-gadget")
        .style = "";
    } else {
      gadget.props.element.querySelector("div.file-upload")
        .style = "";
      gadget.props.element.querySelector("div.editor-gadget")
        .style = "display:none;";
    }
  }

  function displayDataInCodeEditor(gadget, code_editor, url, type) {
    return RSVP.Queue()
      .push(function () {
        return gadget.crib_sw_get(url);
      })
      .push(function (data) {
        return jIO.util.readBlobAsText(data, type);
      })
      .push(function (evt) {
        return code_editor.render({
          key: CODE_CONTENT_KEY,
          value: evt.target.result,
          mode: type
        });
      })
      .push(function () {
        gadget.props.use_editor = true;
        displayProperEditor(gadget);
      });
  }

  function displayFile(gadget, data, type) {
    gadget.props.element
                .querySelector("form.crib-editor-file-upload").reset();
    gadget.props.use_editor = false;
    displayProperEditor(gadget);
  }

  function getUrlTextContent(gadget, event, url) {
    var type = mimeType.lookup(url, false, "application/octet-stream"),
        code_editor;
    gadget.props.element.querySelector("form.crib-editor-save .mimetype")
      .value = type;
    return new RSVP.Queue()
    .push(function () {
      return gadget.getDeclaredGadget('codeeditor');
    })
    .push(function (editor) {
      code_editor = editor;
      return editor.getSupportedTypeList();
    })
    .push(function (result_list) {
      var data = result_list[1];
      if (result_list.indexOf(type) !== -1) {
        return displayDataInCodeEditor(gadget, code_editor, url, type);
      } else {
        return displayFile(gadget, data, type);
      }
    })
    .push(undefined, function (error) {
      gadget.props.element.querySelector(".crib-editor-save-status")
        .textContent = error;
    });
  }

  rJS(window)
    .ready(function (g) {
      g.props = {};
      return g.getElement()
        .push(function (element) {
          g.props.element = element;
          g.props.use_editor = true;
          g.props.start_deferred = RSVP.defer();
        });
    })
    .allowPublicAcquisition("notifySubmit", function () {
      return saveTextContent(this, undefined);
    })
    .allowPublicAcquisition("notifyChange", function () {
      return;
    })
    .declareAcquiredMethod("crib_sw_getScope", "crib_sw_getScope")
    .declareAcquiredMethod("crib_sw_get", "crib_sw_get")
    .declareAcquiredMethod("crib_sw_put", "crib_sw_put")
    .declareAcquiredMethod(
      "crib_sw_setCribEnableGadgetUrl",
      "crib_sw_setCribEnableGadgetUrl"
    )
    .declareAcquiredMethod("setSetting", "setSetting")
    .declareAcquiredMethod("getUrlFor", "getUrlFor")
    .declareAcquiredMethod("redirect", "redirect")
    .declareMethod('render', function (options) {
      var gadget = this;
      if (options === undefined)
        options = {};
      gadget.props.options = options;
      if (gadget.props.options.message !== undefined) {
        gadget.props.element.querySelector(".crib-editor-save-status")
                .textContent = gadget.props.options.message;
      }
      return new RSVP.Queue()
        .push(function () {
          if (gadget.props.options.crib_enable_url !== undefined) {
            return gadget.crib_sw_setCribEnableGadgetUrl(
              gadget.props.options.crib_enable_url
            );
          }
          return;
        })
        .push(function () {
          if (options.url !== undefined) {
            gadget.props.element.querySelector("form.crib-editor-get .url").value = options.url;
            return RSVP.all([
              getUrlTextContent(gadget, undefined, options.url),
              gadget.crib_sw_getScope()
            ]);
          }
        })
        .push(function (result_list) {
          var url_list_scope_link;
          url_list_scope_link = gadget.props.element
            .querySelector("a.url-list-scope");
          gadget.props.scope = result_list[1];
          url_list_scope_link.href = gadget.props.scope;
          url_list_scope_link.innerHTML = gadget.props.scope
            .split("//")[1].split(".")[0];
          return gadget.props.start_deferred.resolve();
        });
    })
    .declareService(function () {
      var gadget = this;
      return new RSVP.Queue()
        .push(function () {
          return gadget.props.start_deferred.promise;
        })
        .push(function () {
          var promise_list = [];
          // promise to get content from URL
          promise_list.push(loopEventListener(
            gadget.props.element.querySelector(".crib-editor-get"),
            'submit',
            false,
            function (event) {
              var url = gadget.props.element.querySelector("form.crib-editor-get .url").value;
              if (event.submitter.name === "go") {
                return gadget.redirect({
                  url: gadget.props.scope + url,
                  newTab: true
                });
              }
              return gadget.redirect({page: "editor", url: url});
            }
          ));
          // promise to save content
          promise_list.push(loopEventListener(
            gadget.props.element.querySelector(".crib-editor-save"),
            'submit',
            false,
            function (event) {saveTextContent(gadget, event); }
          ));
          return RSVP.all(promise_list);
        });
    });
}(window, rJS, loopEventListener, jIO, mimeType));