/*global window, rJS, loopEventListener, RSVP, console, document, WebTorrent,
  File, setInterval, clearInterval, mimeType, URL*/
/*jslint nomen: true, indent: 2, maxerr: 3*/
(function (window, rJS, loopEventListener, JSZip, WebTorrent,
           mimeType) {
  "use strict";

  var CRIBJS_URL = "https://cribjs.nexedi.net/";

  function log(gadget, str, klass) {
    var p = document.createElement('p'),
      old_p_element = gadget.props.element.querySelector(klass + ' p'),
      log_element = gadget.props.element.querySelector(klass);
    if (old_p_element) {
      old_p_element.remove();
    }
    p.innerHTML = str;
    log_element.style.display = "";
    log_element.appendChild(p);
  }

  function logExportZipFile(gadget, str) {
    return log(gadget, str, ".crib-save-to-zip-status");
  }

  function logExportZipTorrent(gadget, str) {
    return log(gadget, str, ".crib-export-to-zip-torrent-status");
  }

  // Zip Methods
  function prepareZip(gadget, event, options, path_to_save, logFunction) {
    var path_to_save_length, crib_sw_gadget,
      url_list = [], final_url_list = [], saved_number = 0, zip;
    if (options === undefined) {
      options = {};
    }
    if (options.type === undefined) {
      options.type = "blob";
    }
    path_to_save_length = path_to_save.length;
    logFunction(gadget, "Preparing");
    return new RSVP.Queue()
    .push(function () {
      return gadget.crib_sw_allDocs({cached_only: true});
    })
    .push(function (data) {
      var promise_list = [],
        i, i_len, url;
      if (data.hasOwnProperty("urls")) {
        data = data.urls;
      } else {
        data = data;
      }
      if (Array.isArray(data)) {
        url_list = data;
      } else {
        url_list = Object.keys(data);
      }
      for (i = 0, i_len = url_list.length; i < i_len; i += 1) {
        url = String(url_list[i]);
        if (url.indexOf(path_to_save) === 0) {
          saved_number += 1;
          promise_list.push(gadget.crib_sw_get(url));
          final_url_list.push(url);
        }
      }
      logFunction(gadget, "Retrieving " + saved_number + " Files");
      return RSVP.all(promise_list);
    })
    .push(function (response_list) {
      var promise_list = [],
        i, i_len, url, response, extension, zip;
      zip = new JSZip();
      for (i = 0, i_len = response_list.length; i < i_len; i += 1) {
        response = response_list[i];
        url = final_url_list[i].substr(path_to_save_length);
        if (url.endsWith("//")) {
          url = url.substr(0, url.length - 1);
        }
        if (url.endsWith("/./")) {
          url = url.substr(0, url.length - 2);
        }
        if (url.endsWith("/")) {
          url = url + "index.html";
        }
        if (url.startsWith("./")) {
          url = url.substr(1);
        }
        zip.file(url, response);
      }
      logFunction(gadget, "Preparing Zip");
      return zip.generateAsync({type: options.type});
    });
  }

  function formatSizeUnits(bytes) {
    if      (bytes >= 1073741824) { bytes = (bytes / 1073741824).toFixed(2) + " GB"; }
    else if (bytes >= 1048576)    { bytes = (bytes / 1048576).toFixed(2) + " MB"; }
    else if (bytes >= 1024)       { bytes = (bytes / 1024).toFixed(2) + " KB"; }
    else if (bytes > 1)           { bytes = bytes + " bytes"; }
    else if (bytes == 1)          { bytes = bytes + " byte"; }
    else                          { bytes = "0 bytes"; }
    return bytes;
  }

  function saveContentToZIP(gadget, event) {
    var path_to_save = gadget.props.element
          .querySelector('form.crib-save-to-zip .save-zip-path').value;
    gadget.props.element.querySelector(".crib-save-to-zip-link").style.display = "none";
    return prepareZip(gadget, event, {}, path_to_save, logExportZipFile)
      .push(function (content) {
        logExportZipFile(
          gadget,
          "Saved a " + formatSizeUnits(content.size) + " zip file at " + Date()
        );
        gadget.props.element.querySelector(".crib-save-to-zip-link a")
          .href = URL.createObjectURL(content);
        gadget.props.element.querySelector(".crib-save-to-zip-link").style.display = "";
      })
      .push(console.log, console.log);
  }

  function displayCribLink(gadget, magnet_uri) {
    new RSVP.Queue()
      .push(function () {
        return gadget.getUrlFor({
          page: "add_application",
          magnet_link: magnet_uri
        });
      })
      .push(function (url) {
        var link = gadget.props.element.querySelector("div.magnet-link a");
        url = CRIBJS_URL + url;
        link.href = url;
        link.innerHTML = url;
      });
  }

  function seedZip(gadget, event) {
    var path_to_save = gadget.props.element
          .querySelector('form.crib-export-torrent .torrent-save-zip-path')
          .value;
    return prepareZip(gadget, event, {}, path_to_save, logExportZipTorrent)
      .push(function (result) {
        logExportZipTorrent(
          gadget,
          "Sharing a " + formatSizeUnits(result.size) + " zip file at " + Date()
        );
        gadget.client = new WebTorrent();
        gadget.client.seed(new File([result], "data.zip"), function (torrent) {
          gadget.props.element.querySelector("div.magnet-link").style.display = "";
          gadget.props.element
            .querySelector("textarea.magnet-link").textContent = torrent.magnetURI;
          console.log('Client is seeding ' + torrent.magnetURI);
          displayCribLink(gadget, torrent.magnetURI);
        });
      });
  }

  rJS(window)
    .ready(function (g) {
      g.props = {};
      return g.getElement()
        .push(function (element) {
          g.props.element = element;
          g.props.start_deferred = RSVP.defer();
        });
    })
    .declareAcquiredMethod("crib_sw_allDocs", "crib_sw_allDocs")
    .declareAcquiredMethod("crib_sw_get", "crib_sw_get")
    .declareAcquiredMethod("getUrlFor", "getUrlFor")
    .declareAcquiredMethod(
      "crib_sw_setCribEnableGadgetUrl",
      "crib_sw_setCribEnableGadgetUrl"
    )
    .declareAcquiredMethod("crib_sw_getScope", "crib_sw_getScope")
    .declareMethod('render', function (options) {
      var gadget = this;
      if (options === undefined)
        options = {};
      gadget.props.options = options;
      return new RSVP.Queue()
        .push(function () {
          if (gadget.props.options.crib_enable_url !== undefined) {
            return gadget.crib_sw_setCribEnableGadgetUrl(
              gadget.props.options.crib_enable_url
            );
          }
          return;
        })
        .push(function () {
          return gadget.crib_sw_getScope();
        })
        .push(function (scope) {
          var url_list_scope_link;
          url_list_scope_link = gadget.props.element
            .querySelector("a.url-list-scope");
          url_list_scope_link.href = scope;
          url_list_scope_link.innerHTML = scope
            .split("//")[1].split(".")[0];
          return gadget.props.start_deferred.resolve();
        });
    })
    .declareService(function () {
      var gadget = this;
      return new RSVP.Queue()
        .push(function () {
          return gadget.props.start_deferred.promise;
        })
        .push(function () {
          if (gadget.props.options.zip_download_path) {
            gadget.props.element
              .querySelector('form.crib-save-to-zip .save-zip-path')
              .value = gadget.props.options.zip_download_path;
          }
          if (gadget.props.options.start_zip_download) {
            return saveContentToZIP(gadget, undefined);
          } else if (gadget.props.options.start_seeding) {
            return seedZip(gadget, undefined);
          }
        });
    })
    .declareService(function () {
      var gadget = this;
      return new RSVP.Queue()
        .push(function () {
          return gadget.props.start_deferred.promise;
        })
        .push(function () {
          var promise_list = [];
          // promise to save content to ZIP
          promise_list.push(loopEventListener(
            gadget.props.element.querySelector(".crib-save-to-zip"),
            'submit',
            false,
            function (event) {saveContentToZIP(gadget, event); }
          ));
          promise_list.push(loopEventListener(
            gadget.props.element.querySelector(".crib-export-torrent"),
            'submit',
            false,
            function (event) {seedZip(gadget, event); }
          ));
          return RSVP.all(promise_list);
        });
    });
}(window, rJS, loopEventListener, JSZip, WebTorrent, mimeType));