/*global window, rJS, loopEventListener, RSVP, console, document */
/*jslint nomen: true, indent: 2, maxerr: 3*/
(function (window, rJS, loopEventListener) {
  "use strict";

  function displayURLList(gadget, event) {
    var url_list = {},
      pattern = gadget.props.element.querySelector(".search-pattern").value,
      scope;
    return new RSVP.Queue()
    .push(function () {
      return RSVP.all([
        gadget.crib_sw_allDocs(),
        gadget.crib_sw_getScope()
      ]);
    })
    .push(function (result_list) {
      var data = result_list[0],
        promise_list = [],
        tmp_url_list = [],
        url_list_scope_link,
        url;
      scope = result_list[1];
      url_list_scope_link = gadget.props.element
        .querySelector("a.url-list-scope");
      url_list_scope_link.href = scope;
      url_list_scope_link.innerHTML = scope.split("//")[1].split(".")[0];
      if (data.hasOwnProperty("urls")) {
        tmp_url_list = data.urls;
      } else {
        tmp_url_list = data;
      }
      for (url in tmp_url_list) {
        if (tmp_url_list.hasOwnProperty(url)) {
          if (!pattern || (pattern && url.indexOf(pattern) != -1)) {
            url_list[url] = "";
            promise_list.push(RSVP.all([
              gadget.getUrlFor({page: 'editor', url: url}),
              gadget.getUrlFor({page: 'mass_remove', url: url})
            ]));
          }
        }
      }
      return RSVP.all(promise_list);
    })
    .push(function (url_link_list) {
      var contentsElement = gadget.props.element.querySelector('.crib-url-list-content tbody'),
        footer_element = gadget.props.element.querySelector('.crib-url-list-content tfoot'),
        url_number = 0, cached_number = 0,
        url, trElement, tdElement;
      // Clear out the existing items from the list.
      while (contentsElement.firstChild) {
        contentsElement.removeChild(contentsElement.firstChild);
      }
      console.log(url_list);
      // Add each cached URL to the list, one by one.
      for (url in url_list) {
        if (url_list.hasOwnProperty(url)) {
          var element;
          trElement = document.createElement('tr');
          tdElement = document.createElement('td');
          element = document.createElement('a');
          element.setAttribute('href', url_link_list[url_number][0]);
          element.textContent = url;
          tdElement.appendChild(element);
          trElement.appendChild(tdElement);
          tdElement = document.createElement('td');
          element = document.createElement('a');
          element.textContent = "Edit";
          element.setAttribute('href', url_link_list[url_number][0]);
          element.setAttribute("class", "btn btn-default btn-xs");
          tdElement.appendChild(element);
          trElement.appendChild(tdElement);
          tdElement = document.createElement('td');
          element = document.createElement('a');
          element.innerHTML = "&#x274c;";
          element.setAttribute('href', url_link_list[url_number][1]);
          element.setAttribute("class", "btn btn-xs");
          tdElement.appendChild(element);
          trElement.appendChild(tdElement);
          tdElement = document.createElement('td');
          element = document.createElement('a');
          element.textContent = "Go";
          element.setAttribute('href', scope + url);
          element.setAttribute('target', "_blank");
          element.setAttribute("class", "btn btn-primary btn-xs");
          tdElement.appendChild(element);
          trElement.appendChild(tdElement);
          contentsElement.appendChild(trElement);
          cached_number = url_list[url].cached ? cached_number + 1 : cached_number;
          url_number += 1;
        }
      }
      while (footer_element.firstChild) {
        footer_element.removeChild(footer_element.firstChild);
      }
      trElement = document.createElement('tr');
      tdElement = document.createElement('td');
      tdElement.innerHTML = "<span>" + cached_number + " &#x2713;</span>";
      trElement.appendChild(tdElement);
      tdElement = document.createElement('td');
      tdElement.innerHTML =  url_number + " URLs";
      trElement.appendChild(tdElement);
      tdElement = document.createElement('td');
      trElement.appendChild(tdElement);
      footer_element.appendChild(trElement);
    });
  }

  rJS(window)
    .ready(function (g) {
      g.props = {};
      return g.getElement()
        .push(function (element) {
          g.props.element = element;
          g.props.start_deferred = RSVP.defer();
        });
    })
    .declareAcquiredMethod("crib_sw_allDocs", "crib_sw_allDocs")
    .declareAcquiredMethod("crib_sw_getScope", "crib_sw_getScope")
    .declareAcquiredMethod(
      "crib_sw_setCribEnableGadgetUrl",
      "crib_sw_setCribEnableGadgetUrl"
    )
    .declareAcquiredMethod("getUrlFor", "getUrlFor")
    .declareAcquiredMethod("redirect", "redirect")
    .declareAcquiredMethod("setSetting", "setSetting")
    .declareAcquiredMethod("getSetting", "getSetting")
    .declareMethod('render', function (options) {
      var gadget = this;
      if (options === undefined)
        options = {};
      gadget.props.options = options;
      return new RSVP.Queue()
        .push(function () {
          if (gadget.props.options.crib_enable_url !== undefined) {
            return gadget.crib_sw_setCribEnableGadgetUrl(
              gadget.props.options.crib_enable_url
            );
          }
          return;
        })
        .push(function () {
          if (options.pattern !== undefined) {
            return gadget.setSetting("editor-pattern", options.pattern)
              .push(function () {
                return options.pattern;
              });
          }
          return gadget.getSetting("editor-pattern", "");
        })
        .push(function (pattern) {
          gadget.props.element.querySelector(".search-pattern", options.pattern)
            .value = pattern;
          return displayURLList(gadget, undefined);
        })
        .push(function () {
          return gadget.props.start_deferred.resolve();
        });
    })
    .declareService(function () {
      var gadget = this;
      return new RSVP.Queue()
        .push(function () {
          return gadget.props.start_deferred.promise;
        })
        .push(function () {
          return loopEventListener(
            gadget.props.element.querySelector(".crib-url-list-content"),
            'submit',
            false,
            function (event) {
              if (event.submitter.name === "add") {
                return gadget.redirect({
                  page: "add",
                  url: gadget.props.element.querySelector(".search-pattern")
                             .value
                });
              }
              return gadget.redirect({
                page: "url_list",
                pattern: gadget.props.element.querySelector(".search-pattern")
                  .value
              });
            }
          );
        });
    });
}(window, rJS, loopEventListener));